package com.huimv.guowei.admin.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnvDevice;
import com.huimv.guowei.admin.entity.EnvDeviceEquipment;
import com.huimv.guowei.admin.mqtt.EnvControll.PublishMQTTTree;
import com.huimv.guowei.admin.service.IEnvDeviceEquipmentService;
import com.huimv.guowei.admin.service.IEnvDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 环控绑定设备 前端控制器
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@RestController
@RequestMapping("/env-device-equipment")
@CrossOrigin
@Slf4j
public class EnvDeviceEquipmentController {
    @Autowired
    private IEnvDeviceEquipmentService equipmentService;
    @Autowired
    private IEnvDeviceService deviceService;

    @PostMapping("/update")
    public Result update(@RequestBody EnvDeviceEquipment envDeviceEquipment) {
        try {
            Date date = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            System.out.println("----这个是开始请求的时间："+sdf.format(date));
            equipmentService.updateById(envDeviceEquipment);
            EnvDeviceEquipment byId = equipmentService.getById(envDeviceEquipment.getId());
            String deviceCode = byId.getDeviceCode();
            System.out.println("设备" + deviceCode);
            QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", deviceCode);
            EnvDevice one = deviceService.getOne(queryWrapper);
            String chipId = one.getDeviceCode();
            log.info("-------获取设备编码：" + chipId);
            log.info("-----开始配置消息----");
            QueryWrapper<EnvDeviceEquipment> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_code", deviceCode).orderByDesc("equipment_port");
            List<EnvDeviceEquipment> list = equipmentService.list(queryWrapper1);
            StringBuilder deviceStatus = new StringBuilder();
            for (EnvDeviceEquipment envDeviceEquipment1 : list) {
                deviceStatus.append(envDeviceEquipment1.getUploadStart());
            }
            String s = deviceStatus.toString();
            int i = Integer.parseInt(s, 2);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("dev_ctrl", i);
            log.info("消息配置完成开始发送：" + jsonObject.toJSONString());
            PublishMQTTTree publishMQTTTree = new PublishMQTTTree();
            Boolean start = publishMQTTTree.start("tcp://115.238.57.190:1883", "GW" + chipId, "admin", "admin", "huimv_down_" + chipId, jsonObject.toJSONString());


            if (start) {
                Thread.sleep(3000);//线程等待3秒
                EnvDeviceEquipment byId1 = equipmentService.getById(envDeviceEquipment.getId());
                Date updateTime = byId1.getUpdateTime();
                System.out.println("-----这个是修改后的时间："+updateTime);
                int i1 = updateTime.compareTo(date);
                if (i1 > 0) {
                    QueryWrapper<EnvDeviceEquipment> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.eq("device_code", deviceCode).orderByDesc("equipment_port");
                    List<EnvDeviceEquipment> list1 = equipmentService.list(queryWrapper2);
                    for (EnvDeviceEquipment equipment : list1) {
                        equipmentService.updateById(equipment);
                    }
                    return new Result(10000, "修改成功", true);
                } else {
                    if (envDeviceEquipment.getUploadStart() == 1) {
                        envDeviceEquipment.setUploadStart(0);
                    } else {
                        envDeviceEquipment.setUploadStart(1);
                    }
                    equipmentService.updateById(envDeviceEquipment);
                    return new Result(10001, "设备掉线，请等待设备重连", false);
                }
            } else {
                if (envDeviceEquipment.getUploadStart() == 1) {
                    envDeviceEquipment.setUploadStart(0);
                } else {
                    envDeviceEquipment.setUploadStart(1);
                }
                equipmentService.updateById(envDeviceEquipment);
                return new Result(10001, "网络连接超时，请等待5秒后重试", false);
            }

        } catch (Exception e) {
            System.out.println("22222222222" + e);
            return new Result(10001, "修改失败", false);
        }

    }


    @PostMapping("/getStatus")
    public Result getStatus(){
        double rate = 0.0;
        Map resultMap = new HashMap();
        int total = equipmentService.count();
        int deviceOn = equipmentService.count(new QueryWrapper<EnvDeviceEquipment>().eq("open_start",1));
        int deviceOff = equipmentService.count(new QueryWrapper<EnvDeviceEquipment>().eq("open_start",0));
        resultMap.put("totalDevice", total);
        resultMap.put("deviceOn", deviceOn);
        resultMap.put("deviceOff", deviceOff);
        if (deviceOff == 0 && deviceOn != 0){
            rate = 100;
        }
        if (deviceOff != 0){
            rate = (deviceOn/total)*100;
        }
        resultMap.put("onlineRate",rate);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @PostMapping("/listDevice")
    public Result listDevice(Map<String, String> paramsMap){
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<EnvDeviceEquipment> page = new Page(Integer.valueOf(pageNum),Integer.valueOf(pageSize));
        return new Result(ResultCode.SUCCESS,equipmentService.page(page,new QueryWrapper<EnvDeviceEquipment>().orderByAsc("unit_id").orderByAsc("equipment_type")));
    }


    @PostMapping("/listScreen")
    public Result listScreen(HttpServletRequest httpServletRequest,@RequestBody Map<String, String> paramsMap){
        return equipmentService.listScreen(httpServletRequest, paramsMap);
    }
}
