package com.huimv.guowei.admin.mqtt.EnvControll;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.entity.EnvWarningInfo;
import com.huimv.guowei.admin.service.IEnvWarningInfoService;
import com.huimv.guowei.admin.utils.SpringUtil;
import com.huimv.guowei.admin.entity.EnvDevice;
import com.huimv.guowei.admin.entity.EnvDeviceEquipment;
import com.huimv.guowei.admin.service.IEnvDeviceEquipmentService;
import com.huimv.guowei.admin.service.IEnvDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.ZonedDateTime;
import java.util.*;

@Component
//接收消息回调
@Slf4j
public class PushCallback implements MqttCallbackExtended {

    @Override
    public void connectComplete(boolean b, String s) {
        System.out.println("重新订阅！");

    }
    @Override
    public void connectionLost(Throwable cause) {
        // 连接丢失后，一般在这里面进行重连
        System.out.println("连接断开，可以做重连");

    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        System.out.println("deliveryComplete---------" + token.isComplete());
    }

    @Override
    public void messageArrived(String topic, MqttMessage message) throws Exception {
        // subscribe后得到的消息会执行到这里面
        System.out.println("接收消息主题 : " + topic);
        System.out.println("接收消息Qos : " + message.getQos());
        System.out.println("接收消息内容 : " + new String(message.getPayload()));
        System.out.println("开始处理当前数据...");

        String[] split = topic.split("_");
        String chip = split[2];
        System.out.println(chip);
        String messages = new String(message.getPayload());
        if ("sync_time".equals(messages)) {
            log.info("-----开始配置同步时间----");
            JSONObject jsonObject1 = new JSONObject();
            ZonedDateTime ZonedDateTime = java.time.ZonedDateTime.now();
            Map<String, Object> map = new HashMap<>();
            map.put("year", ZonedDateTime.getYear());
            map.put("mon", ZonedDateTime.getMonthValue());
            map.put("date", ZonedDateTime.getDayOfMonth());
            map.put("hour", ZonedDateTime.getHour());
            map.put("min", ZonedDateTime.getMinute());
            map.put("sec", ZonedDateTime.getSecond());
            jsonObject1.put("sync_time", map);
            System.out.println(jsonObject1);
            try {
                PublishMQTT publishMQTT = new PublishMQTT();
                publishMQTT.start("tcp://115.238.57.190:1883", "GUOWEIpublish", "admin", "admin", "huimv_down_" + chip, jsonObject1.toJSONString());
            } catch (Exception e) {
                System.out.println(e);
            }
        }else if ("alarm_cfg".equals(messages)) {
            log.info("高低温报警不做处理！！");
        } else if ("get_dev_ctrl".equals(messages)) {
            JSONObject jsonObject = new JSONObject();
            log.info("-----开始配置设备控制消息----");
            QueryWrapper<EnvDevice> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_code", chip);
            IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
            EnvDevice one = envDeviceService.getOne(queryWrapper1);

            IEnvDeviceEquipmentService envDeviceEquipmentService = SpringUtil.getBean(IEnvDeviceEquipmentService.class);
            QueryWrapper<EnvDeviceEquipment> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("device_code", one.getId()).orderByDesc("equipment_port");
            List<EnvDeviceEquipment> list = envDeviceEquipmentService.list(queryWrapper);
            String deviceStatus = "";
            for (EnvDeviceEquipment envDeviceEquipment1 : list) {
                deviceStatus = deviceStatus + envDeviceEquipment1.getUploadStart();
            }
            String s = deviceStatus;
            int i = Integer.parseInt(s, 2);
            jsonObject.put("dev_ctrl", i);
            log.info("设备配置完成开始发送：" + i);
            PublishMQTT publishMQTT = new PublishMQTT();
            publishMQTT.start("tcp://115.238.57.190:1883", "GUOWEIpublish", "admin", "admin", "huimv_down_" + chip, jsonObject.toJSONString());
            log.info("---设备控制发送完成---");
        }  else if ("dev_ctrl_suc".equals(messages)) {
            QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("device_code", chip);
            IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
            EnvDevice one = envDeviceService.getOne(queryWrapper);

            IEnvDeviceEquipmentService envDeviceEquipmentService = SpringUtil.getBean(IEnvDeviceEquipmentService.class);
            QueryWrapper<EnvDeviceEquipment> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_code", one.getId()).orderByDesc("equipment_port");
            List<EnvDeviceEquipment> list = envDeviceEquipmentService.list(queryWrapper1);
            for (EnvDeviceEquipment equipment : list) {
                equipment.setUpdateTime(new Date());
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                System.out.println("----这个是应答时间："+sdf.format(new Date()));
                envDeviceEquipmentService.updateById(equipment);
                System.out.println("-----收到控制响应后修改标识成功！！");
            }

            System.out.println("收到控制响应");
        } else {
            JSONObject jsonObject = JSON.parseObject(messages);
            System.out.println("这里是1111：" + jsonObject);
             if (ObjectUtil.isNotEmpty(jsonObject.get("alarm_event"))) {
                log.info("----报警信息开始保存---");
                JSONObject jsonObject1 = (JSONObject) jsonObject.get("alarm_event");
                try {
                    IEnvWarningInfoService envWarningInfoService = SpringUtil.getBean(IEnvWarningInfoService.class);
                    QueryWrapper<EnvDevice> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("device_code", chip);
                    IEnvDeviceService envDeviceService = SpringUtil.getBean(IEnvDeviceService.class);
                    EnvDevice one = envDeviceService.getOne(queryWrapper);

                    EnvWarningInfo warningInfo = new EnvWarningInfo();
                    warningInfo.setWarningType(4);
                    warningInfo.setBuildLocation(one.getBuildLocation());
                    warningInfo.setUnitId(one.getUnitId());
                    warningInfo.setDate(new Date());
                    warningInfo.setFarmId(one.getFarmId());
                    Integer type = (Integer) jsonObject1.get("sta");
                    if (type == 1) {
                        warningInfo.setWarningContent(one.getBuildLocation()+"的设备断电报警！");
                    } else {
                        warningInfo.setWarningContent(one.getBuildLocation()+"的设备恢复供电！");
                    }
                    envWarningInfoService.save(warningInfo);
                } catch (Exception e) {
                    System.out.println("报警信息保存异常！！！");
                }

                log.info("----报警信息保存成功！----");
            }
        }

        System.out.println("数据处理完成！");
    }



}
