package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseDuckInfo;
import com.huimv.guowei.admin.entity.DuckHealthInfo;
import com.huimv.guowei.admin.entity.DuckImmunityInfo;
import com.huimv.guowei.admin.mapper.BaseDuckInfoMapper;
import com.huimv.guowei.admin.mapper.DuckImmunityInfoMapper;
import com.huimv.guowei.admin.service.IDuckImmunityInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class DuckImmunityInfoServiceImpl extends ServiceImpl<DuckImmunityInfoMapper, DuckImmunityInfo> implements IDuckImmunityInfoService {

    @Autowired
    private DuckImmunityInfoMapper infoMapper;
    @Autowired
    private BaseDuckInfoMapper baseDuckInfoMapper;
    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String duckNum = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");

        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }

        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        if (startDate == null || "".equals(startDate)) {
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).orderByAsc("health_date");
        }else {
            startDate = startDate + " 00:00:00";
            endDate = endDate + " 23:59:59";
            queryWrapper.eq("duck_id", duckNum).eq("farm_id", farmId).between("health_date", startDate, endDate).orderByAsc("health_date");
        }
        Page<DuckImmunityInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, infoMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result addImmunity(HttpServletRequest httpServletRequest, DuckImmunityInfo duckImmunityInfo) throws ParseException {

        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("duck_num", duckImmunityInfo.getDuckNum());
        BaseDuckInfo baseDuckInfo = baseDuckInfoMapper.selectOne(queryWrapper);
        duckImmunityInfo.setDuckId(baseDuckInfo.getUnitId());
        duckImmunityInfo.setDuckNum(baseDuckInfo.getDuckNum());
        infoMapper.insert(duckImmunityInfo);
        return new Result(ResultCode.SUCCESS);
    }
    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("health_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, infoMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<DuckImmunityInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName),"unit_name", unitName)
        .orderByDesc("health_date");
        Page<DuckImmunityInfo> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, infoMapper.selectPage(page, queryWrapper));
    }

}
