package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnergyElectricity;
import com.huimv.guowei.admin.entity.EnergyWater;
import com.huimv.guowei.admin.mapper.EnergyElectricityMapper;
import com.huimv.guowei.admin.service.IEnergyElectricityService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.xml.crypto.Data;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnergyElectricityServiceImpl extends ServiceImpl<EnergyElectricityMapper, EnergyElectricity> implements IEnergyElectricityService {
    @Resource
    private EnergyElectricityMapper energyElectricityMapper;

    @Override
    public Result getElectricity(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Map resultMap = new HashMap();
        BigDecimal dayElectricity = energyElectricityMapper.getDayElectricity(farmId);
        if (dayElectricity == null){
            dayElectricity = BigDecimal.ZERO;
        }
        BigDecimal monthElectricity = energyElectricityMapper.getMonthElectricity(farmId);
        if (monthElectricity == null){
            monthElectricity = BigDecimal.ZERO;
        }
        resultMap.put("dayElectricity",dayElectricity);
        resultMap.put("MonthElectricity",monthElectricity);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listElectricity(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";

        List<EnergyElectricity> energyElectricityList = null;
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listDayDetail(farmId);
        }
        if ("2".equals(type)){
            energyElectricityList = energyElectricityMapper.listWeekDetail(farmId);
        }
        if ("3".equals(type)){
            energyElectricityList = energyElectricityMapper.listMonthDetail(farmId);
        }
        if ("4".equals(type)){
            energyElectricityList = energyElectricityMapper.listCustomDetail(farmId,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }

    @Override
    public Result getScreenElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        Date timesMonthmorning = DataUill.getTimesMonthmorning();
        BigDecimal dayGap,monthGap;
        Map resultMap = new HashMap();
        //今日
        QueryWrapper<EnergyElectricity> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesmorning);
        BigDecimal dayElectricity = energyElectricityMapper.getElectricity(queryWrapper1);
        //昨日
        LocalDate yesterday = LocalDate.now().minusDays(1);
        LocalDateTime startOfDay = yesterday.atStartOfDay();
        LocalDateTime endOfDay = yesterday.atTime(LocalTime.MAX);
        QueryWrapper<EnergyElectricity> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",startOfDay,endOfDay);
        BigDecimal lastDayElectricity = energyElectricityMapper.getElectricity(queryWrapper2);
        //日环比
        if (lastDayElectricity.toString().equals("0.0")){
            dayGap = BigDecimal.valueOf(100);
            if (dayElectricity.toString().equals("0.0")){
                dayGap = BigDecimal.valueOf(0);
            }
        }else {
            dayGap = dayElectricity.subtract(lastDayElectricity).divide(lastDayElectricity,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }

        //本月
        QueryWrapper<EnergyElectricity> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesMonthmorning);
        BigDecimal monthElectricity = energyElectricityMapper.getElectricity(queryWrapper3);
        //上月
        Date lastMonthStartMorning = DataUill.getLastMonthStartMorning();
        Date lastMonthEndNight = DataUill.getLastMonthEndNight();
        QueryWrapper<EnergyElectricity> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",lastMonthStartMorning,lastMonthEndNight);
        BigDecimal lastMonthElectricity = energyElectricityMapper.getElectricity(queryWrapper4);
        //月环比
        System.out.println(monthElectricity);
        System.out.println(lastMonthElectricity);
        if (lastMonthElectricity.toString().equals("0.0")){
            monthGap = BigDecimal.valueOf(100);
            if (monthElectricity.toString().equals("0.0")){
                monthGap = BigDecimal.valueOf(0);
            }
        }else {
            monthGap = monthElectricity.subtract(lastMonthElectricity).divide(lastMonthElectricity,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }
        int dayTrend =dayElectricity.compareTo(lastDayElectricity);
        int monthTrend =monthElectricity.compareTo(lastMonthElectricity);
        resultMap.put("dayElectricity",dayElectricity);
        resultMap.put("dayTrend",dayTrend);
        resultMap.put("dayElectricityGap",dayGap+"%");
        resultMap.put("monthElectricity",monthElectricity);
        resultMap.put("monthTrend",monthTrend);
        resultMap.put("monthElectricityGap",monthGap+"%");
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listScreenElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        List<EnergyElectricity> energyElectricityList;
        //六个月
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listSixMonthElectricity(farmId, unitId);
        }
        //七天
        else {
            energyElectricityList = energyElectricityMapper.listSevenDayElectricity(farmId, unitId);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }

    @Override
    public Result listUnitElectricity(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        List<EnergyElectricity> energyElectricityList;
        //日
        if ("1".equals(type)){
            energyElectricityList = energyElectricityMapper.listUnitDayElectricity(farmId, unitId);
        }
        //月
        else {
            energyElectricityList = energyElectricityMapper.listUnitMonthElectricity(farmId, unitId);
        }
        return new Result(ResultCode.SUCCESS,energyElectricityList);
    }
}
