package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnergyElectricity;
import com.huimv.guowei.admin.entity.EnergyWater;
import com.huimv.guowei.admin.entity.EnvDeviceChip;
import com.huimv.guowei.admin.entity.vo.WaterLineVo;
import com.huimv.guowei.admin.mapper.EnergyWaterMapper;
import com.huimv.guowei.admin.mapper.EnvDeviceChipMapper;
import com.huimv.guowei.admin.service.IEnergyWaterService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnergyWaterServiceImpl extends ServiceImpl<EnergyWaterMapper, EnergyWater> implements IEnergyWaterService {
    @Resource
    private EnergyWaterMapper energyWaterMapper;
    @Resource
    private EnvDeviceChipMapper envDeviceChipMapper;

    @Override
    public Result listWater(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";

        List<EnergyWater> energyWaterList = null;
        if ("1".equals(type)){
            energyWaterList = energyWaterMapper.listDayDetail(farmId);
        }
        if ("2".equals(type)){
            energyWaterList = energyWaterMapper.listWeekDetail(farmId);
        }
        if ("3".equals(type)){
            energyWaterList = energyWaterMapper.listMonthDetail(farmId);
        }
        if ("4".equals(type)){
            energyWaterList = energyWaterMapper.listCustomDetail(farmId,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,energyWaterList);
    }

    @Override
    public Result getWater(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Map resultMap = new HashMap();
        BigDecimal dayWater = energyWaterMapper.getDayWater(farmId);
        if (dayWater == null){
            dayWater = BigDecimal.ZERO;
        }
        BigDecimal monthWater = energyWaterMapper.getMonthWater(farmId);
        if (monthWater == null){
            monthWater = BigDecimal.ZERO;
        }
        resultMap.put("dayWater",dayWater);
        resultMap.put("MonthWater",monthWater);
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listWaterLine(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        if (StringUtils.isEmpty(unitId)){
            unitId = "232";
        }
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";
        List<WaterLineVo> waterLineList = new ArrayList<>();
        List<EnvDeviceChip> deviceChipList = envDeviceChipMapper.selectList(new QueryWrapper<EnvDeviceChip>().eq("unit_id",unitId).eq("chip_type", 3));
        if (ObjectUtil.isEmpty(deviceChipList)){
            return new Result(ResultCode.SUCCESS,waterLineList);
        }
        List<String> chipCodeList = new ArrayList<>();
        for (EnvDeviceChip envDeviceChip : deviceChipList) {
            chipCodeList.add(envDeviceChip.getChipCode());
        }
        String chipCodes = String.join(",", chipCodeList);

        chipCodes = "("+chipCodes+")";
        if ("1".equals(type)){
            waterLineList = energyWaterMapper.listWaterLineDayDetail(farmId,chipCodeList,chipCodes);
        }
        if ("2".equals(type)){
            waterLineList = energyWaterMapper.listWaterLineWeekDetail(farmId,chipCodeList,chipCodes);
        }
        if ("3".equals(type)){
            waterLineList = energyWaterMapper.listWaterLineMonthDetail(farmId,chipCodeList,chipCodes);
        }
        if ("4".equals(type)){
            waterLineList = energyWaterMapper.listWaterLineCustomDetail(farmId,chipCodeList,chipCodes,startDate,endDate);
        }

        return new Result(ResultCode.SUCCESS,waterLineList);
    }

    @Override
    public Result getScreenWaterLine(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        Date timesMonthmorning = DataUill.getTimesMonthmorning();
        BigDecimal dayGap,monthGap;
        Map resultMap = new HashMap();
        //今日
        QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesmorning);
        BigDecimal dayWater = energyWaterMapper.getWater(queryWrapper1);
        //昨日
        LocalDate yesterday = LocalDate.now().minusDays(1);
        LocalDateTime startOfDay = yesterday.atStartOfDay();
        LocalDateTime endOfDay = yesterday.atTime(LocalTime.MAX);
        QueryWrapper<EnergyWater> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",startOfDay,endOfDay);
        BigDecimal lastDayWater = energyWaterMapper.getWater(queryWrapper2);

        System.out.println(lastDayWater);
        //日环比
        if (lastDayWater.toString().equals("0.0")){
            dayGap = BigDecimal.valueOf(100);
            if (dayWater.toString().equals("0.0")){
                dayGap = BigDecimal.valueOf(0);
            }
        }else {
            dayGap = dayWater.subtract(lastDayWater).divide(lastDayWater).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }
        //本月
        QueryWrapper<EnergyWater> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("unit_id",unitId).eq("farm_id",farmId).ge("create_date",timesMonthmorning);
        BigDecimal monthWater = energyWaterMapper.getWater(queryWrapper3);
        //上月
        Date lastMonthStartMorning = DataUill.getLastMonthStartMorning();
        Date lastMonthEndNight = DataUill.getLastMonthEndNight();
        QueryWrapper<EnergyWater> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("unit_id",unitId).eq("farm_id",farmId).between("create_date",lastMonthStartMorning,lastMonthEndNight);
        BigDecimal lastMonthWater = energyWaterMapper.getWater(queryWrapper4);
        //月环比
        if (lastMonthWater.toString().equals("0.0")){
            monthGap = BigDecimal.valueOf(100);
            if (monthWater.toString().equals("0.0")){
                monthGap = BigDecimal.valueOf(0);
            }
        }else {
            monthGap = monthWater.subtract(lastMonthWater).divide(lastMonthWater,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }
        int dayTrend =dayWater.compareTo(lastDayWater);
        int monthTrend =monthWater.compareTo(lastMonthWater);
        resultMap.put("dayWater",dayWater);
        resultMap.put("dayTrend",dayTrend);
        resultMap.put("dayWaterGap",dayGap+"%");
        resultMap.put("monthWater",monthWater);
        resultMap.put("monthTrend",monthTrend);
        resultMap.put("monthWaterGap",monthGap+"%");
        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listScreenWaterLine(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        String unitId = paramsMap.get("unitId");
        List<WaterLineVo> waterLineList = new ArrayList<>();
        List<EnvDeviceChip> deviceChipList = envDeviceChipMapper.selectList(new QueryWrapper<EnvDeviceChip>().eq("unit_id",unitId).eq("chip_type", 3));
        if (ObjectUtil.isEmpty(deviceChipList)){
            return new Result(ResultCode.SUCCESS,waterLineList);
        }
        List<String> chipCodeList = new ArrayList<>();
        for (EnvDeviceChip envDeviceChip : deviceChipList) {
            chipCodeList.add(envDeviceChip.getChipCode());
        }
        String chipCodes = String.join(",", chipCodeList);

        chipCodes = "("+chipCodes+")";
        //六个月
        if ("1".equals(type)){
            waterLineList = energyWaterMapper.listSixMonthWaterLine(farmId, chipCodeList,chipCodes);
        }
        //七天
        else {
            waterLineList = energyWaterMapper.listSevenDayWaterLine(farmId, chipCodeList,chipCodes);
        }
        return new Result(ResultCode.SUCCESS,waterLineList);
    }

    @Override
    public Result listScreenUnitWater(Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        List<EnergyWater> energyWaterList = new ArrayList<>();
        if ("1".equals(type)){
            energyWaterList = energyWaterMapper.listUnitDayDetail(farmId,unitId);
        }else {
            energyWaterList = energyWaterMapper.listUnitMonthDetail(farmId,unitId);
        }
        return new Result(ResultCode.SUCCESS,energyWaterList);
    }
}
