package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.BaseBuilding;
import com.huimv.guowei.admin.entity.EnvData;
import com.huimv.guowei.admin.entity.vo.EnvDataVo;
import com.huimv.guowei.admin.mapper.BaseBuildingMapper;
import com.huimv.guowei.admin.mapper.EnvDataMapper;
import com.huimv.guowei.admin.service.IEnvDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvDataServiceImpl extends ServiceImpl<EnvDataMapper, EnvData> implements IEnvDataService {

    @Resource
    private EnvDataMapper envDataMapper;

    @Resource
    private BaseBuildingMapper baseBuildingMapper;

    @Override
    public Result getLastFlow(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        List<EnvDataVo> envDataList = envDataMapper.getLastFlow(farmId);
        return new Result(ResultCode.SUCCESS,envDataList);
    }

    @Override
    public Result listByBuild(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate")+" 23:59:59";
        String type = paramsMap.get("type");

        List<EnvData> envDataList = null;
//      默认情况:第一个栋舍的本周数据
        if (StringUtils.isBlank(type)){
            List<BaseBuilding> baseBuildings = baseBuildingMapper.selectList(new QueryWrapper<BaseBuilding>().eq("parent_id", 0));
            if (ObjectUtil.isEmpty(baseBuildings)){
                return new Result(10001,"请先添加栋舍!",false);
            }
            unitId = String.valueOf(baseBuildings.get(0).getId());
            envDataList = envDataMapper.listWeekDetail(farmId,unitId);
        }

        //今日查询
        if ("1".equals(type)){
            envDataList = envDataMapper.listDayDetail(farmId,unitId);
        }
        //本周查询
        if ("2".equals(type)){
            envDataList = envDataMapper.listWeekDetail(farmId,unitId);
        }
        //本月查询
        if ("3".equals(type)){
            envDataList = envDataMapper.listMonthDetail(farmId,unitId);
        }
        //自定义查询
        if ("4".equals(type)){
            envDataList = envDataMapper.listCustomDetail(farmId,unitId,startDate,endDate);
        }
        return new Result(ResultCode.SUCCESS,envDataList);
    }

}
