package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.DuckImmunityInfo;
import com.huimv.guowei.admin.entity.EnvMoveCall;
import com.huimv.guowei.admin.entity.EnvRegularCallEgg;
import com.huimv.guowei.admin.mapper.EnvMoveCallMapper;
import com.huimv.guowei.admin.service.IEnvMoveCallService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvMoveCallServiceImpl extends ServiceImpl<EnvMoveCallMapper, EnvMoveCall> implements IEnvMoveCallService {

    @Autowired
    private EnvMoveCallMapper moveCallMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//显示类型
        String duckCode = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        if ("1".equals(type)) {
            QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_id", duckCode).orderByAsc("dayage").eq("farm_id", farmId);
            Page<EnvMoveCall> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, moveCallMapper.selectPage(page, queryWrapper));
        } else {
            QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId);
            String startAge = paramsMap.get("startAge");
            String endAge = paramsMap.get("endAge");
            if (startAge == null || "".equals(startAge)) {
                queryWrapper.eq("duck_id", duckCode);
            } else {
                queryWrapper.eq("duck_id", duckCode).between("dayage", startAge, endAge);
            }
            return new Result(ResultCode.SUCCESS, moveCallMapper.listDay(queryWrapper));
        }

    }

    @Override
    public Result listWeight(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
        if (startDate == null || "".equals(startDate)) {
            queryWrapper.eq("farm_id", farmId).orderByAsc("call_date");
        } else {
            startDate = startDate + " 00:00:00";
            endDate = endDate + " 23:59:59";
            queryWrapper.eq("farm_id", farmId).between("call_date", startDate, endDate).orderByAsc("call_date");
        }
        Page<EnvMoveCall> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, moveCallMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId)
                .like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .orderByDesc("call_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, moveCallMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String type = paramsMap.get("type");//显示类型
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        QueryWrapper<EnvMoveCall> queryWrapper = new QueryWrapper<>();
        if ("1".equals(type)) {
            queryWrapper.like(StringUtils.isNotBlank(unitName), "unit_name", unitName).orderByDesc("call_date").eq("farm_id", farmId);
            Page<EnvMoveCall> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, moveCallMapper.selectPage(page, queryWrapper));
        } else {
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
            queryWrapper.eq("farm_id", farmId)
                    .like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                    .ge("call_date", calendar.getTime());
            return new Result(ResultCode.SUCCESS, moveCallMapper.listDay(queryWrapper));
        }
    }
}
