package com.huimv.guowei.admin.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.mapper.*;
import com.huimv.guowei.admin.service.IEnvRegularCallEggService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.guowei.admin.utils.Print;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 固定电子秤 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvRegularCallEggServiceImpl extends ServiceImpl<EnvRegularCallEggMapper, EnvRegularCallEgg> implements IEnvRegularCallEggService {

    @Autowired
    private EnvRegularCallEggMapper eggMapper;
    @Autowired
    private BaseDuckInfoMapper infoMapper;
    @Autowired
    private EnvRegularCallFeedingMapper feedingMapper;
    @Autowired
    private EnvDataMapper dataMapper;
    @Autowired
    private EnergyElectricityMapper electricityMapper;
    @Autowired
    private EnergyWaterMapper waterMapper;
    @Autowired
    private BaseBuildingMapper buildingMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//显示类型
        String duckCode = paramsMap.get("id");//鸭子的id
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        if ("1".equals(type)) {
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_id", duckCode).eq("farm_id", farmId);
            Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, eggMapper.listPage(page, queryWrapper));
        } else {
            String dateType = paramsMap.get("dateType");
            QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("duck_id", duckCode);
            if ("1".equals(dateType)) {
                Date timesmorning = DataUill.getTimesmorning();
                queryWrapper.ge("call_date", timesmorning);

            } else if ("2".equals(dateType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                queryWrapper.ge("call_date", calendar.getTime());

            } else if ("3".equals(dateType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                queryWrapper.ge("call_date", timesMonthmorning);

            } else {
                String startDate = paramsMap.get("startDate");
                String endDate = paramsMap.get("endDate");
                startDate = startDate + " 00:00:00";
                endDate = endDate + " 23:59:59";
                queryWrapper.between("call_date", startDate, endDate);
            }
            return new Result(ResultCode.SUCCESS, eggMapper.listDay(queryWrapper));
        }
    }

    @Override
    public Result addEgg(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("duckId");
        QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", duckId);
        BaseDuckInfo baseDuckInfo = infoMapper.selectOne(queryWrapper);
        String date = paramsMap.get("date");
        date = date + " 00:00:00";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String eggNum = paramsMap.get("eggNum");
        if (Integer.parseInt(eggNum) < 0) {
            return new Result(10001, "蛋的个数不能为负数", false);
        }
        String eggWeight = paramsMap.get("eggWeight");
        BigDecimal weight = new BigDecimal(eggWeight);
        if (weight.compareTo(BigDecimal.ZERO) != 1) {
            return new Result(10001, "蛋的重量必须大于0", false);
        }
        EnvRegularCallEgg egg = new EnvRegularCallEgg();
        egg.setDuckNum(baseDuckInfo.getDuckNum());
        egg.setFarmId(Integer.parseInt(farmId));
        egg.setDuckWeight(Double.valueOf(eggWeight));
        egg.setEggNum(Integer.parseInt(eggNum));
        egg.setCallDate(sdf.parse(date));
        egg.setDuckId(Integer.parseInt(duckId));
        eggMapper.insert(egg);
        return new Result(10000, "添加成功!", true);
    }

    @Override
    public Result updateEgg(HttpServletRequest httpServletRequest, EnvRegularCallEgg egg) {
        eggMapper.updateById(egg);
        return new Result(10000, "修改成功！", true);
    }

    @Override
    public Result deleteEgg(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");//此时这里就是表中的id
        eggMapper.deleteById(id);
        return new Result(10000, "删除成功!", true);
    }

    @Override
    public void printEgg(HttpServletResponse response, Map<String, String> paramsMap) throws Exception {
        String duckCode = paramsMap.get("duckCode");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("duck_num", duckCode);
        List<EnvRegularCallEgg> eggs = eggMapper.printEggList(queryWrapper);
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("duck_num", duckCode);
        List<EnvRegularCallFeeding> feedings = feedingMapper.listDay(queryWrapper1);
        Print.printEgg(eggs, feedings);
    }

    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId)
                .like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .orderByDesc("call_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, eggMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }

        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .select("IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum,duck_num duckNum,unit_name unitName,duck_id duckId")
                .groupBy("duck_num")
                .orderByDesc("call_date");
        Page<EnvRegularCallEgg> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, eggMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listSeven(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("id");
        QueryWrapper<EnvRegularCallEgg> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        queryWrapper.eq("farm_id", farmId).eq(StringUtils.isNotBlank(duckId), "duck_id", duckId)
        /* .ge("call_date", calendar.getTime())*/;
        QueryWrapper<EnvRegularCallEgg> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq(StringUtils.isNotBlank(duckId), "duck_id", duckId);
        queryWrapper1.select(" IFNULL(SUM(duck_weight),'0') duckWeight, IFNULL(SUM(egg_num),'0') eggNum");
        EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper1);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("eggNum", egg.getEggNum());
        jsonObject.put("egg", eggMapper.listDay(queryWrapper));
        jsonObject.put("eggWeight", egg.getDuckWeight());
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result collector(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String duckNum = paramsMap.get("duckNum");//鸭只编号
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");//传入的是某个舍的id
        QueryWrapper<BaseBuilding> buildingQueryWrapper = new QueryWrapper<>();
        buildingQueryWrapper.eq("farm_id", farmId).eq("parent_id", unitId);
        List<BaseBuilding> baseBuildings = buildingMapper.selectList(buildingQueryWrapper);//某个舍下所有的笼子

        JSONArray jsonArray = new JSONArray();

        for (BaseBuilding baseBuilding : baseBuildings) {
            Integer baseBuildingId = baseBuilding.getId();//笼子id
            QueryWrapper<BaseDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("unit_id", baseBuildingId).like(StringUtils.isNotBlank(duckNum), "duck_num", duckNum);
            BaseDuckInfo baseDuckInfo = infoMapper.selectOne(queryWrapper);//笼子里的鸭只信息

            if (ObjectUtil.isNotEmpty(baseDuckInfo)) {
                QueryWrapper<EnvRegularCallEgg> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("farm_id", farmId).eq("duck_num", baseDuckInfo.getDuckNum());
                queryWrapper1.select("SUM(egg_num) eggNum,duck_num duckNum");
                queryWrapper1.groupBy("duck_num");
                EnvRegularCallEgg egg = eggMapper.selectOne(queryWrapper1);//鸡蛋个数

                QueryWrapper<EnvRegularCallFeeding> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("farm_id", farmId).eq("duck_num", baseDuckInfo.getDuckNum());
                queryWrapper2.select("SUM(duck_weight) duckWeight,duck_num duckNum");
                queryWrapper2.groupBy("duck_num");
                EnvRegularCallFeeding envRegularCallFeeding = feedingMapper.selectOne(queryWrapper2);//采食量


                JSONObject jsonObject = new JSONObject();
                if (ObjectUtil.isEmpty(egg)) {
                    jsonObject.put("egg", 0);
                } else {
                    jsonObject.put("egg", egg.getEggNum());
                }
                if (ObjectUtil.isEmpty(envRegularCallFeeding)) {
                    jsonObject.put("feed", 0);
                } else {
                    jsonObject.put("feed", envRegularCallFeeding.getDuckWeight());
                }
                jsonObject.put("duckNum", baseDuckInfo.getDuckNum());
                jsonObject.put("location", baseDuckInfo.getUnitName());
                jsonArray.add(jsonObject);
            }

        }


        return new Result(ResultCode.SUCCESS, jsonArray);
    }

    @Override
    public Result collectorEnv(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_date").last(" limit 1");
        EnergyElectricity energyElectricity = electricityMapper.selectOne(queryWrapper);//电表

        QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_date").last(" limit 1");
        EnergyWater water = waterMapper.selectOne(queryWrapper1);//水表

        QueryWrapper<EnvData> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("farm_id", farmId).eq("unit_id", unitId).orderByDesc("create_time").last(" limit 1");
        EnvData envData = dataMapper.selectOne(queryWrapper2);//温湿度

        JSONObject jsonObject = new JSONObject();
        if (ObjectUtil.isEmpty(energyElectricity)) {
            jsonObject.put("electricity", 0);
        } else {
            jsonObject.put("electricity", energyElectricity.getElectricityValue());
        }
        if (ObjectUtil.isEmpty(water)) {
            jsonObject.put("water", 0);
        } else {
            jsonObject.put("water", water.getWaterValue());
        }
        if (ObjectUtil.isEmpty(envData)) {
            jsonObject.put("hum", 0);
            jsonObject.put("temp", 0);
        } else {
            jsonObject.put("hum", envData.getEnvHum());
            jsonObject.put("temp", envData.getEnvTemp());
        }

        return new Result(ResultCode.SUCCESS, jsonObject);
    }

}
