package com.huimv.guowei.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.*;
import com.huimv.guowei.admin.mapper.EnvRegularCallFeedingMapper;
import com.huimv.guowei.admin.service.IEnvRegularCallFeedingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * <p>
 * 固定电子秤 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@Service
public class EnvRegularCallFeedingServiceImpl extends ServiceImpl<EnvRegularCallFeedingMapper, EnvRegularCallFeeding> implements IEnvRegularCallFeedingService {

    @Autowired
    private EnvRegularCallFeedingMapper feedingMapper;

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//显示类型
        String duckCode = paramsMap.get("id");
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        if ("1".equals(type)) {
            QueryWrapper<EnvRegularCallFeeding> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("duck_id", duckCode).eq("farm_id", farmId);
            Page<EnvRegularCallFeeding> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS,feedingMapper.listPage(page, queryWrapper));
        } else {
            String dateType = paramsMap.get("dateType");
            QueryWrapper<EnvRegularCallFeeding> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", farmId).eq("duck_id", duckCode);
            if ("1".equals(dateType)) {
                Date timesmorning = DataUill.getTimesmorning();
                queryWrapper.ge("call_date", timesmorning);

            } else if ("2".equals(dateType)) {
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
                queryWrapper.ge("call_date", calendar.getTime());

            } else if ("3".equals(dateType)) {
                Date timesMonthmorning = DataUill.getTimesMonthmorning();
                queryWrapper.ge("call_date", timesMonthmorning);

            } else {
                String startDate = paramsMap.get("startDate");
                String endDate = paramsMap.get("endDate");
                startDate = startDate + " 00:00:00";
                endDate = endDate + " 23:59:59";
                queryWrapper.between("call_date", startDate, endDate);
            }
            return new Result(ResultCode.SUCCESS,feedingMapper.listDay(queryWrapper));
        }

    }
    @Override
    public Result listByH5(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        QueryWrapper<EnvRegularCallFeeding> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId)
                .like(StringUtils.isNotBlank(unitName),"unit_name", unitName).orderByDesc("call_date").last(" limit 10");
        return new Result(ResultCode.SUCCESS, feedingMapper.selectList(queryWrapper));
    }

    @Override
    public Result listSelect(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitName = paramsMap.get("unitName");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if (pageNum == null || "".equals(pageNum)) {
            pageNum = "1";
        }
        if (pageSize == null || "".equals(pageSize)) {
            pageSize = "10";
        }
        Date timesMonthmorning = DataUill.getTimesmorning();
        QueryWrapper<EnvRegularCallFeeding> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(unitName), "unit_name", unitName)
                .select("SUM(duck_weight) duckWeight,duck_num duckNum,call_date callDate,unit_name unitName,duck_id duckId")
                .ge("call_date", timesMonthmorning)
                .orderByDesc("call_date");
        Page<EnvRegularCallFeeding> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, feedingMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listSeven(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String duckId = paramsMap.get("id");
        QueryWrapper<EnvRegularCallFeeding> queryWrapper = new QueryWrapper<>();
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        queryWrapper.eq("farm_id", farmId).eq("event_type",0)
                .eq(StringUtils.isNotBlank(duckId),"duck_id", duckId)
                .ge("call_date", calendar.getTime());
        return new Result(ResultCode.SUCCESS, feedingMapper.listDay(queryWrapper));
    }

    @Override
    public Result getScreenFeed(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        Date timesmorning = DataUill.getTimesmorning();
        Date timesMonthmorning = DataUill.getTimesMonthmorning();
        BigDecimal dayGap,monthGap;
        Map resultMap = new HashMap();
        //今日
        QueryWrapper<EnvRegularCallFeeding> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("d.unit_id",unitId).eq("f.farm_id",farmId).eq("f.event_type",0).ge("f.call_date",timesmorning);
        BigDecimal dayFeed = feedingMapper.getFeed(queryWrapper1);
        //昨日
        LocalDate yesterday = LocalDate.now().minusDays(1);
        LocalDateTime startOfDay = yesterday.atStartOfDay();
        LocalDateTime endOfDay = yesterday.atTime(LocalTime.MAX);
        QueryWrapper<EnvRegularCallFeeding> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("d.unit_id",unitId).eq("f.farm_id",farmId).eq("f.event_type",0).between("f.call_date",startOfDay,endOfDay);
        BigDecimal lastDayFeed = feedingMapper.getFeed(queryWrapper2);
        //日环比
        if (lastDayFeed.toString().equals("0.00")){
            dayGap = BigDecimal.valueOf(100);
            if (dayFeed.toString().equals("0.00")){
                dayGap = BigDecimal.valueOf(0);
            }
        }else {
            dayGap = dayFeed.subtract(lastDayFeed).divide(lastDayFeed,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }
        //本月
        QueryWrapper<EnvRegularCallFeeding> queryWrapper3 = new QueryWrapper<>();
        queryWrapper3.eq("d.unit_id",unitId).eq("f.farm_id",farmId).eq("f.event_type",0).ge("f.call_date",timesMonthmorning);
        BigDecimal monthFeed = feedingMapper.getFeed(queryWrapper3);
        //上月
        Date lastMonthStartMorning = DataUill.getLastMonthStartMorning();
        Date lastMonthEndNight = DataUill.getLastMonthEndNight();
        QueryWrapper<EnvRegularCallFeeding> queryWrapper4 = new QueryWrapper<>();
        queryWrapper4.eq("d.unit_id",unitId).eq("f.farm_id",farmId).eq("f.event_type",0).between("f.call_date",lastMonthStartMorning,lastMonthEndNight);
        BigDecimal lastMonthFeed = feedingMapper.getFeed(queryWrapper4);
        //月环比
        if (lastMonthFeed.toString().equals("0.00")){
            monthGap = BigDecimal.valueOf(100);
            if (monthFeed.toString().equals("0.00")){
                monthGap = BigDecimal.valueOf(0);
            }
        }else {
            monthGap = monthFeed.subtract(lastMonthFeed).divide(lastMonthFeed,BigDecimal.ROUND_HALF_UP).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.UP);
        }
        int dayTrend =dayFeed.compareTo(lastDayFeed);
        int monthTrend =monthFeed.compareTo(lastMonthFeed);
        resultMap.put("dayFeed",dayFeed);
        resultMap.put("dayTrend",dayTrend);
        resultMap.put("dayFeedGap",dayGap+"%");
        resultMap.put("monthFeed",monthFeed);
        resultMap.put("monthTrend",monthTrend);
        resultMap.put("monthFeedGap",monthGap+"%");

        return new Result(ResultCode.SUCCESS,resultMap);
    }

    @Override
    public Result listScreenFeed(Map<String, String> paramsMap) {
        String unitId = paramsMap.get("unitId");
        String farmId = paramsMap.get("farmId");
        String type = paramsMap.get("type");
        List<EnvRegularCallFeeding> envRegularCallFeedingList;
        //六个月
        if ("1".equals(type)){
            envRegularCallFeedingList = feedingMapper.listSixMonthFeed(farmId, unitId);
        }
        //七天
        else {
            envRegularCallFeedingList = feedingMapper.listSevenDayFeed(farmId, unitId);
        }
        return new Result(ResultCode.SUCCESS,envRegularCallFeedingList);
    }

}
